# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import os
import sys
import pytest

os.chdir(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(r'../'))


if __name__ == r'__main__':

    pytest.main()



# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import asyncio

import pytest

from hagworm.extend.error import Ignore
from hagworm.extend.asyncio.base import Utils, ShareFuture, async_adapter
from hagworm.extend.asyncio.base import AsyncConstructor, AsyncCirculatory, AsyncCirculatoryForSecond
from hagworm.extend.asyncio.base import AsyncFuncWrapper, FuncCache
from hagworm.extend.asyncio.future import FutureWithTimeout
from hagworm.extend.interface import AsyncContextManager
from hagworm.extend.asyncio.transaction import Transaction


pytestmark = pytest.mark.asyncio
# pytest.skip(allow_module_level=True)


class TestUtils:

    async def test_context_manager_1(self):

        class _ContextManager(AsyncContextManager):
            async def _context_release(self):
                pass

        result = False

        try:

            async with _ContextManager():
                raise Ignore()

            result = True

        except Ignore:

            result = False

        return result

    async def test_context_manager_2(self):

        class _ContextManager(AsyncContextManager):
            async def _context_release(self):
                pass

        result = False

        try:

            async with _ContextManager():

                async with _ContextManager():
                    raise Ignore(layers=2)

                result = False

            result = True

        except Ignore:

            result = False

        return result

    async def test_share_future(self):

        @ShareFuture()
        async def _do_acton():
            await Utils.sleep(1)
            return Utils.randint(0, 0xffff)

        res_1 = await _do_acton()
        await Utils.sleep(0)
        res_2 = await _do_acton()

        assert res_1 != res_2

        tasks = []

        for _ in range(0xff):
            tasks.append(_do_acton())

        result = await asyncio.gather(*tasks)

        assert len(set(result)) == 1

    async def test_async_adapter_and_wait_frame(self):

        result = False

        async def _temp():
            nonlocal result
            result = True
            await Utils.wait_frame(0xf0)

        async_adapter(_temp)()

        await Utils.wait_frame(0xff)

        assert result

    async def test_future_with_timeout(self):

        await FutureWithTimeout(1)

    async def test_async_constructor(self):

        result = False

        class _Temp(AsyncConstructor):

            async def __async_init__(self):
                nonlocal result
                result = True

        temp = await _Temp()

        assert isinstance(temp, _Temp)
        assert result

    async def test_async_circulatory_1(self):

        async for _ in AsyncCirculatory(1, 0xff):
            pass

    async def test_async_circulatory_2(self):

        index = 0

        async for index in AsyncCirculatory(0, 0xff, 0xff):
            pass
        else:
            assert index == 0xff

    async def test_async_circulatory_for_second_1(self):

        index = 0

        async for index in AsyncCirculatoryForSecond(1, 0.1):
            pass

        assert (index >= 9) and (index <= 11)

    async def test_async_circulatory_for_second_2(self):

        index = 0

        async for index in AsyncCirculatoryForSecond(0, 0.1, 10):
            pass
        else:
            assert index == 10

    async def test_async_context_manager(self):

        result = False

        class _Temp(AsyncContextManager):

            async def _context_release(self):
                nonlocal result
                result = True
                await Utils.wait_frame(0xff)

        async with _Temp() as _:
            pass

        assert result

    async def test_async_func_wrapper(self):

        result1 = False
        result2 = False

        async def _temp1():
            nonlocal result1
            result1 = True
            await Utils.wait_frame(0xff)

        async def _temp2():
            nonlocal result2
            result2 = True
            await Utils.wait_frame(0xff)

        wrapper = AsyncFuncWrapper()

        wrapper.add(_temp1)
        wrapper.add(_temp2)

        await wrapper()

        assert result1 and result2

    async def test_transaction(self):

        result = []

        def _temp1():
            nonlocal result
            result.append(True)

        async def _temp2():
            nonlocal result
            result.append(True)
            await Utils.wait_frame(0xff)

        async with Transaction() as trx1:
            trx1.add_commit_callback(_temp1)
            trx1.add_commit_callback(_temp2)
            await trx1.commit()

        assert all(result) and len(result) == 2

        async with Transaction() as trx2:
            trx2.add_rollback_callback(_temp1)
            trx2.add_rollback_callback(_temp2)
            await trx1.rollback()

        assert all(result) and len(result) == 4

        async with Transaction() as trx3:
            trx3.add_rollback_callback(_temp1)
            trx3.add_rollback_callback(_temp2)

        assert all(result) and len(result) == 6

    async def test_func_cache(self):

        @FuncCache(ttl=1)
        async def _do_acton():
            return Utils.randint(0, 0xffff)

        res1 = await _do_acton()
        res2 = await _do_acton()

        assert res1 == res2

        await Utils.sleep(1)
        res3 = await _do_acton()

        assert res1 != res3



# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import pytest

from hagworm.extend.asyncio.base import Utils
from hagworm.extend.struct import KeyLowerDict


pytestmark = pytest.mark.asyncio
# pytest.skip(allow_module_level=True)


class TestStruct:

    async def test_key_lower_dict(self):

        temp1 = {
            r'MySQL1': {
                r'MySQL1': Utils.randint(0x10, 0xff),
                r'MYSQL2': Utils.randint(0x10, 0xff),
                r'Mysql3': Utils.randint(0x10, 0xff),
            },
            r'MYSQL2': {
                r'MySQL1': {
                    r'MySQL1': Utils.randint(0x10, 0xff),
                    r'MYSQL2': Utils.randint(0x10, 0xff),
                    r'Mysql3': Utils.randint(0x10, 0xff),
                },
                r'MYSQL2': {
                    r'MySQL1': Utils.randint(0x10, 0xff),
                    r'MYSQL2': Utils.randint(0x10, 0xff),
                    r'Mysql3': Utils.randint(0x10, 0xff),
                },
                r'Mysql3': {
                    r'MySQL1': Utils.randint(0x10, 0xff),
                    r'MYSQL2': Utils.randint(0x10, 0xff),
                    r'Mysql3': Utils.randint(0x10, 0xff),
                },
            },
            r'Mysql3': {
                r'MySQL1': {
                    r'MySQL1': {
                        r'MySQL1': Utils.randint(0x10, 0xff),
                        r'MYSQL2': Utils.randint(0x10, 0xff),
                        r'Mysql3': Utils.randint(0x10, 0xff),
                    },
                    r'MYSQL2': {
                        r'MySQL1': Utils.randint(0x10, 0xff),
                        r'MYSQL2': Utils.randint(0x10, 0xff),
                        r'Mysql3': Utils.randint(0x10, 0xff),
                    },
                    r'Mysql3': {
                        r'MySQL1': Utils.randint(0x10, 0xff),
                        r'MYSQL2': Utils.randint(0x10, 0xff),
                        r'Mysql3': Utils.randint(0x10, 0xff),
                    },
                },
                r'MYSQL2': {
                    r'MySQL1': {
                        r'MySQL1': Utils.randint(0x10, 0xff),
                        r'MYSQL2': Utils.randint(0x10, 0xff),
                        r'Mysql3': Utils.randint(0x10, 0xff),
                    },
                    r'MYSQL2': {
                        r'MySQL1': Utils.randint(0x10, 0xff),
                        r'MYSQL2': Utils.randint(0x10, 0xff),
                        r'Mysql3': Utils.randint(0x10, 0xff),
                    },
                    r'Mysql3': {
                        r'MySQL1': Utils.randint(0x10, 0xff),
                        r'MYSQL2': Utils.randint(0x10, 0xff),
                        r'Mysql3': Utils.randint(0x10, 0xff),
                    },
                },
                r'Mysql3': {
                    r'MySQL1': {
                        r'MySQL1': Utils.randint(0x10, 0xff),
                        r'MYSQL2': Utils.randint(0x10, 0xff),
                        r'Mysql3': Utils.randint(0x10, 0xff),
                    },
                    r'MYSQL2': {
                        r'MySQL1': Utils.randint(0x10, 0xff),
                        r'MYSQL2': Utils.randint(0x10, 0xff),
                        r'Mysql3': Utils.randint(0x10, 0xff),
                    },
                    r'Mysql3': {
                        r'MySQL1': Utils.randint(0x10, 0xff),
                        r'MYSQL2': Utils.randint(0x10, 0xff),
                        r'Mysql3': Utils.randint(0x10, 0xff),
                    },
                },
            },
        }

        temp2 = KeyLowerDict(temp1)

        assert r'my_sql1' in temp2 and r'mysql2' in temp2 and r'mysql3' in temp2



# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import time
import pytest

from hagworm.extend.asyncio.future import ThreadWorker


pytestmark = pytest.mark.asyncio
# pytest.skip(allow_module_level=True)


class TestWorker:

    @ThreadWorker(1)
    def _temp_for_thread_worker(self, *args, **kwargs):
        time.sleep(1)
        return True

    async def test_thread_worker(self):

        assert await self._temp_for_thread_worker()
        assert await self._temp_for_thread_worker(1, 2)
        assert await self._temp_for_thread_worker(1, 2, t1=1, t2=2)



# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import pytest

from hagworm.extend.error import catch_error, Ignore
from hagworm.extend.interface import ContextManager
from hagworm.extend.base import FuncWrapper


pytestmark = pytest.mark.asyncio
# pytest.skip(allow_module_level=True)


class TestUtils:

    async def test_catch_error_1(self):

        result = False

        try:

            with catch_error():
                raise Ignore()

            result = True

        except Ignore:

            result = False

        return result

    async def test_catch_error_2(self):

        result = False

        try:

            with catch_error():

                with catch_error():
                    raise Ignore(layers=2)

                result = False

            result = True

        except Ignore:

            result = False

        return result

    async def test_context_manager_1(self):

        class _ContextManager(ContextManager):
            def _context_release(self):
                pass

        result = False

        try:

            with _ContextManager():
                raise Ignore()

            result = True

        except Ignore:

            result = False

        return result

    async def test_context_manager_2(self):

        class _ContextManager(ContextManager):
            def _context_release(self):
                pass

        result = False

        try:

            with _ContextManager():

                with _ContextManager():
                    raise Ignore(layers=2)

                result = False

            result = True

        except Ignore:

            result = False

        return result

    async def test_func_wrapper(self):

        result1 = False
        result2 = False

        def _temp1():
            nonlocal result1
            result1 = True

        def _temp2():
            nonlocal result2
            result2 = True

        wrapper = FuncWrapper()

        wrapper.add(_temp1)
        wrapper.add(_temp2)

        wrapper()

        assert result1 and result2



# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import typing

import grpc
import os
import sys

os.chdir(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(r'../../'))

from pydantic import BaseModel
from pydantic.fields import Field

from hagworm.frame.fastapi.field import Base64Type
from hagworm.third.grpc.server import Router, StreamHandler
from hagworm.third.grpc.bridge import GRPCMainProcess, GRPCWorker


router = Router(r'demo')


class FormTest(BaseModel):

    int_field: int = Field(..., description=r'int_field')
    str_field: Base64Type = Field(..., description=r'str_field')


@router.unary_unary
async def unary_unary_test(request: FormTest, _: grpc.aio.ServicerContext):
    return request.model_dump()


@router.unary_stream
async def unary_stream_test(request: FormTest, context: grpc.aio.ServicerContext):

    for i in range(2):
        await context.write(request.model_dump())


@router.stream_unary
async def stream_unary_test(request: typing.AsyncIterator, _: grpc.aio.ServicerContext):

    messages = []

    async for _message in request:
        messages.append(_message)

    return messages


class StreamStreamTest(StreamHandler):

    async def on_message(self, data: typing.Any):
        await self.write(data)

    async def on_connect(self):
        pass

    def on_close(self, context: grpc.aio.ServicerContext):
        pass

@router.stream_stream
async def stream_stream_test(request: typing.AsyncIterator, context: grpc.aio.ServicerContext):
    await StreamStreamTest(request, context).join()


if __name__ == r'__main__':

    GRPCMainProcess(
        (r'0.0.0.0', 8080), [router],
        GRPCWorker, 1,
        cpu_affinity=True
    ).run()



# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import pytest

from hagworm.extend.asyncio.base import Utils
from hagworm.extend.asyncio.task import MultiTasks, RateLimiter


pytestmark = pytest.mark.asyncio
# pytest.skip(allow_module_level=True)


class TestTasks:

    async def test_multi_tasks(self):

        async def _do_acton(val):
            await Utils.sleep(val)

        tasks = MultiTasks(5)

        tasks.append(_do_acton(8))

        for _ in range(2):

            tasks.append(_do_acton(4))

            for _ in range(2):
                tasks.append(_do_acton(1))
                tasks.append(_do_acton(1))
                tasks.append(_do_acton(2))

        await tasks

    async def test_rate_limiter(self):

        async def _temp(num):
            await Utils.sleep(num)
            return True

        limiter = RateLimiter(2, 5, 200)

        await limiter.append(_temp, 0.1)
        await limiter.append(_temp, 0.2)
        await limiter.append(_temp, 0.3)
        await limiter.append(_temp, 0.4)
        await limiter.append(_temp, 0.5)
        await limiter.append(_temp, 0.6)

        await limiter.join()
        limiter.close()



# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import pytest

from hagworm.extend.asyncio.base import Utils
from hagworm.extend.cache import StackCache


pytestmark = pytest.mark.asyncio
# pytest.skip(allow_module_level=True)


class TestCache:

    async def test_stack_cache(self):

        cache = StackCache(ttl=0.5)

        ckey = Utils.uuid1()
        cval = Utils.uuid1()

        res1 = cache.has(ckey)
        res2 = cache.size()

        assert not res1 and res2 == 0

        cache.set(ckey, cval)

        res3 = cache.has(ckey)
        res4 = cache.get(ckey)
        res5 = cache.size()

        assert res3 and res4 == cval and res5 == 1

        await Utils.sleep(1)

        res6 = cache.has(ckey)
        res7 = cache.get(ckey)
        res8 = cache.size()

        assert not res6 and res7 is None and res8 == 0



# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import os
import sys
import typing
import asyncio
import grpc

os.chdir(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(r'../../'))

from hagworm.extend.asyncio.base import Utils
from hagworm.extend.asyncio.base import install_uvloop
from hagworm.third.grpc.client import GRPCClient, RobustStreamClient


class StreamClient(RobustStreamClient):

    async def on_message(self, data: typing.Any):
        Utils.log.info(str(data))

    async def on_connect(self):
        pass

    def on_close(self, stub: grpc.aio.StreamStreamCall):
        pass



async def main():

    client = GRPCClient()

    await client.open([r'127.0.0.1:8080'])

    Utils.log.info(await client.ping())

    for idx in range(10):
        try:
            Utils.log.info(
                await client.unary_unary(
                    r'/demo/unary_unary_test',
                    {r'int_field': idx, r'str_field': Utils.b64_encode(f'test_{idx}', True)}
                )
            )
        except Exception as err:
            Utils.log.error(str(err))

    ##########

    for idx in range(10):
        _messages = []
        try:
            resp = client.unary_stream(
                r'/demo/unary_stream_test',
                {r'int_field': idx, r'str_field': Utils.b64_encode(f'test_{idx}', True)}
            )
            async for _msg in resp:
                _messages.append(_msg)
            Utils.log.info(_messages)
        except Exception as err:
            Utils.log.error(str(err))

    ##########

    for idx in range(10):
        try:
            resp = client.stream_unary(r'/demo/stream_unary_test')
            for _idx in range(2):
                await resp.write({r'int_field': idx, r'str_field': f'test_{idx}_{_idx}'})
            await resp.done_writing()
            Utils.log.info(await resp)
        except Exception as err:
            Utils.log.error(str(err))

    ##########

    stream = StreamClient(client, r'/demo/stream_stream_test')
    await stream.connect()

    for idx in range(10):
        try:
            await stream.write({'stream': idx})
        except Exception as err:
            Utils.log.error(str(err))
    await stream.done_writing()

    await stream.join()


if __name__ == r'__main__':

    install_uvloop()

    asyncio.run(main())



# -*- coding: utf-8 -*-

__author__ = r'wsb310@gmail.com'

import io
import pytest

from hagworm.extend.asyncio.task import MultiTasks
from hagworm.extend.asyncio.net import HTTPClientPool, HTTPTextClientPool, HTTPJsonClientPool, Downloader


pytestmark = pytest.mark.asyncio
# pytest.skip(allow_module_level=True)

TEST_URLS = [
    r'https://lib.sinaapp.com/js/bootstrap/4.1.3/js/bootstrap.min.js.map',
    r'https://lib.sinaapp.com/js/angular.js/angular-1.2.19/angular.min.js.map',
]


class TestHTTPClient:

    async def _http_client(self, client):

        for url in TEST_URLS:
            response = await client.get(url)
            assert response

    async def _http_client_pool(self, client):

        tasks = MultiTasks()

        for url in TEST_URLS:
            tasks.append(client.get(url))

        await tasks

    async def test_http_download(self):

        downloader = Downloader()
        bytesio = io.BytesIO()

        await downloader.fetch(bytesio, r'https://www.baidu.com/img/flexible/logo/pc/result@2.png')
        await downloader.close()

        assert bytesio.__sizeof__() > 64

    async def test_http_client_pool(self):

        pool = HTTPClientPool()

        await self._http_client(pool)
        await self._http_client_pool(pool)

        await pool.close()

    async def test_http_text_client_pool(self):

        pool = HTTPTextClientPool()

        await self._http_client(pool)
        await self._http_client_pool(pool)

        await pool.close()

    async def test_http_json_client_pool(self):

        pool = HTTPJsonClientPool()

        await self._http_client(pool)
        await self._http_client_pool(pool)

        await pool.close()
